@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID("$Id: clean_request.c,v 1.3 2000/02/24 15:07:39 misiek Exp $")
+RCSID("$Id: clean_request.c,v 1.5 2001/10/17 14:11:54 wiget Exp $")
 
 #include <stdio.h>
 #include <unistd.h>
@@ -29,6 +29,7 @@ RCSID("$Id: clean_request.c,v 1.3 2000/02/24 15:07:39 misiek Exp $")
 #include <syslog.h>
 
 #include "tcpd.h"
+#include "tcpd_local.h"
 
 void clean_request(struct request_info *req)
 {
@@ -46,6 +47,7 @@ void clean_exit(struct request_info *req)
 	clean_request(req);
 	if (!req->udp)
 		sleep(req->refuse_wait);
+	free_request(req);
 	exit(0);
 }
 
