@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID(PKG_VER "$Id: libwrap.c,v 1.20 2000/10/07 16:50:35 baggins Exp $")
+RCSID(PKG_VER "$Id: libwrap.c,v 1.23 2001/10/17 11:47:52 malekith Exp $")
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,7 +56,10 @@ static int ignore_dst_dns_fake;
 static void strtosa(char *str, struct sockaddr_storage *sa)
 {
 	struct addrinfo hints, *res, *res0;
-	
+
+	if (str == NULL || sa == NULL)
+		return;
+
 	memset(sa, 0, sizeof(*sa));
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = AF_UNSPEC;
@@ -75,119 +78,6 @@ static void strtosa(char *str, struct sockaddr_storage *sa)
 	}
 }
 
-#ifdef IN6_IS_ADDR_V4MAPPED
-static void mappedtov4(struct sockaddr_storage *ss)
-{
-	struct sockaddr_in sin;
-	struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)ss;
-
-	if (ss->__ss_family == AF_INET6 &&
-			IN6_IS_ADDR_V4MAPPED(&sin6->sin6_addr) ) {
-		memcpy(&sin.sin_addr, sin6->sin6_addr.s6_addr+12,
-				sizeof(sin.sin_addr));
-		sin.sin_port = ((struct sockaddr_in6 *)ss)->sin6_port;
-		sin.sin_family = AF_INET;
-#ifdef SIN6_LEN
-		sin.sin_len = 16;
-#endif
-		memcpy(ss, &sin, sizeof(sin));
-	}
-}
-#else
-#define        mappedtov4(A)
-#endif
-		
-static struct request_info *request_fill(struct request_info *req, va_list ap)
-{
-	struct sockaddr_storage *ss;
-	int cmd;
-
-	do {
-		cmd = va_arg(ap, int);
-		switch(cmd) {
-		case RQ_FILE:
-			req->sock = va_arg(ap, int);
-			break;
-		case RQ_DAEMON:
-			strncpy(req->service, va_arg(ap, char *), sizeof(req->service));
-			break;
-		case RQ_USER:
-			strncpy(req->user, va_arg(ap, void *), sizeof(req->user));
-			break;
-		case RQ_SERVER_SIN:
-			ss = va_arg(ap, struct sockaddr_storage *);
-			if (ss)
-				memcpy(&req->dst, ss, SA_LEN((struct sockaddr *)ss));
-			break;
-		case RQ_CLIENT_SIN:
-			ss = va_arg(ap, struct sockaddr_storage *);
-			if (ss)
-				memcpy(&req->src, ss, SA_LEN((struct sockaddr *)ss));
-			break;
-		case RQ_SERVER_NAME:
-			strncpy(req->dst_name, va_arg(ap, void *), sizeof(req->dst_name));
-			break;
-		case RQ_CLIENT_NAME:
-			strncpy(req->src_name, va_arg(ap, void *), sizeof(req->src_name));
-			break;
-		case RQ_SERVER_ADDR:
-			strtosa(va_arg(ap, void *), &req->dst);
-			break;
-		case RQ_CLIENT_ADDR:
-			strtosa(va_arg(ap, void *), &req->src);
-			break;
-		case RQ_SETTABLE:
-			strncpy(access_table, va_arg(ap, void *), sizeof(access_table));
-			break;
-		case RQ_END:
-			break;
-		default:
-			cmd = RQ_END;
-			break;
-		}
-	} while(cmd != RQ_END);
-	return (req);
-}
-
-struct request_info *request_init(struct request_info *req, ...)
-{
-	struct request_info *r;
-	va_list ap;
-
-	va_start(ap, req);
-
-	userresolvreq = 1;
-	defaultallow = TCPD_DENY;
-	ignore_src_dns_fake = 0;
-	ignore_dst_dns_fake = 0;
-
-	memset(req, 0, sizeof(struct request_info));
-	req->sock = -1;
-	req->userresolved = 0;
-	req->src.__ss_family = -1;
-	req->dst.__ss_family = -1;
-	allow_severity = deny_severity = LOG_DAEMON | LOG_NOTICE;
-	req->udp = 0;
-	req->refuse_wait = 5;
-	req->src_wrongdns = 0;
-	req->dst_wrongdns = 0;
-	req->rfc1413_timeout = TCPD_DEFAULT_RFC1413_TIMEOUT;
-	r = request_fill(req, ap);
-	va_end(ap);
-	return (r);
-}
-
-struct request_info *request_set(struct request_info *req, ...)
-{
-	struct request_info *r;
-	va_list ap;
-
-	va_start(ap, req);
-	r = request_fill(req, ap);
-	va_end(ap);
-	return(r);
-}
-
 static int gethostaddr(struct sockaddr_storage *sa, char *str, int slen)
 {
 	struct addrinfo hints, *res, *res0 = NULL;
@@ -223,28 +113,130 @@ static int gethostaddr(struct sockaddr_storage *sa, char *str, int slen)
 	return -1;
 }
 
-void fromhost(struct request_info *req)
+#ifdef IN6_IS_ADDR_V4MAPPED
+static void mappedtov4(struct sockaddr_storage *ss)
 {
-	int salen;
+	struct sockaddr_in sin;
+	struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)ss;
 
-	salen = sizeof(req->src);
-	if (getpeername(req->sock, (struct sockaddr *)&req->src, &salen) < 0) {
-		char buf[BUFSIZ];
-		req->udp = 1;
-		salen = sizeof(req->src);
-		if (recvfrom(req->sock, buf, sizeof(buf), MSG_PEEK, (struct sockaddr *)&req->src, &salen) < 0) {
-			return;
+	if (ss->__ss_family == AF_INET6 &&
+			IN6_IS_ADDR_V4MAPPED(&sin6->sin6_addr) ) {
+		memcpy(&sin.sin_addr, sin6->sin6_addr.s6_addr+12,
+				sizeof(sin.sin_addr));
+		sin.sin_port = ((struct sockaddr_in6 *)ss)->sin6_port;
+		sin.sin_family = AF_INET;
+#ifdef SIN6_LEN
+		sin.sin_len = 16;
+#endif
+		memcpy(ss, &sin, sizeof(sin));
+	}
+}
+#else
+#define        mappedtov4(A)
+#endif
+		
+static struct request_info *request_fill(struct request_info *req, int cmd, va_list ap)
+{
+	struct sockaddr_storage *ss;
+
+	if (req == NULL)
+		return NULL;
+
+	for (; cmd != RQ_END; cmd = va_arg(ap, int))  {
+		switch(cmd) {
+		case RQ_FILE:
+			req->sock = va_arg(ap, int);
+			break;
+		case RQ_DAEMON:
+			strncpy(req->service, va_arg(ap, char *), sizeof(req->service));
+			break;
+		case RQ_USER:
+			strncpy(req->user, va_arg(ap, void *), sizeof(req->user));
+			break;
+		case RQ_SERVER_SIN:
+			ss = va_arg(ap, struct sockaddr_storage *);
+			if (ss) {
+				memcpy(&req->dst, ss, SA_LEN((struct sockaddr *)ss));
+				mappedtov4(&req->dst);
+				if (gethostaddr(&req->dst, req->dst_name, sizeof(req->dst_name)) == -1)
+					req->dst_wrongdns = 1;
+			}
+			break;
+		case RQ_CLIENT_SIN:
+			ss = va_arg(ap, struct sockaddr_storage *);
+			if (ss) {
+				memcpy(&req->src, ss, SA_LEN((struct sockaddr *)ss));
+				mappedtov4(&req->src);
+				if (gethostaddr(&req->src, req->src_name, sizeof(req->src_name)) == -1)
+					req->src_wrongdns = 1;
+			}
+			break;
+		case RQ_SERVER_NAME:
+			strncpy(req->dst_name, va_arg(ap, void *), sizeof(req->dst_name));
+			break;
+		case RQ_CLIENT_NAME:
+			strncpy(req->src_name, va_arg(ap, void *), sizeof(req->src_name));
+			break;
+		case RQ_SERVER_ADDR:
+			strtosa(va_arg(ap, void *), &req->dst);
+			break;
+		case RQ_CLIENT_ADDR:
+			strtosa(va_arg(ap, void *), &req->src);
+			break;
+		case RQ_SETTABLE:
+			strncpy(access_table, va_arg(ap, void *), sizeof(access_table));
+			break;
+		case RQ_END:
+			break;
+		default:
+			cmd = RQ_END;
+			break;
 		}
 	}
-	salen = sizeof(req->dst);
-	if (getsockname(req->sock, (struct sockaddr *)&req->dst, &salen) < 0)
-		return;
-	mappedtov4(&req->dst);
-	mappedtov4(&req->src);
-	if (gethostaddr(&req->src, req->src_name, sizeof(req->src_name)) == -1)
-		req->src_wrongdns = 1;
-	if (gethostaddr(&req->dst, req->dst_name, sizeof(req->dst_name)) == -1)
-		req->dst_wrongdns = 1;
+	
+	return (req);
+}
+
+struct request_info *request_init(int cmd, ...)
+{
+	struct request_info *r;
+	va_list ap;
+
+	va_start(ap, cmd);
+
+	userresolvreq = 1;
+	defaultallow = TCPD_DENY;
+	ignore_src_dns_fake = 0;
+	ignore_dst_dns_fake = 0;
+
+	r = malloc(sizeof(struct request_info));
+	if (r == NULL)
+		return NULL;
+	memset(r, 0, sizeof(struct request_info));
+	r->sock = -1;
+	r->userresolved = 0;
+	r->src.__ss_family = -1;
+	r->dst.__ss_family = -1;
+	allow_severity = deny_severity = LOG_DAEMON | LOG_NOTICE;
+	r->udp = 0;
+	r->refuse_wait = 5;
+	r->src_wrongdns = 0;
+	r->dst_wrongdns = 0;
+	r->rfc1413_timeout = TCPD_DEFAULT_RFC1413_TIMEOUT;
+	r = request_fill(r, cmd, ap);
+	va_end(ap);
+	return (r);
+}
+
+struct request_info *request_set(struct request_info *req, int cmd, ...)
+{
+	struct request_info *r;
+	va_list ap;
+
+	va_start(ap, cmd);
+	r = request_fill(req, cmd, ap);
+	va_end(ap);
+	return(r);
 }
 
 /*
@@ -467,8 +459,8 @@ static int host_match(struct request_info *req, char *strp, char **left)
  */
 
 static struct request_info *sreq;
-static struct __service_options ALL_service = {-1, -1, -1, -1, -1, -1, -1, -1, -1};
-static struct __service_options current_service = {-1, -1, -1, -1, -1, -1, -1, -1, -1};
+static struct __service_options ALL_service = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
+static struct __service_options current_service = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
 static struct __service_options *sservicep = &ALL_service;
 
 static int set_syslogallow(char *s)
@@ -663,7 +655,20 @@ static int hosts_access0(FILE *fp, struct request_info *req)
 int hosts_access(struct request_info *req)
 {
 	FILE *fp;
-	int ret;
+	int ret, salen;
+
+	if (req == NULL)
+		return TCPD_DENY;
+
+	salen = sizeof(req->src);
+	if (getpeername(req->sock, (struct sockaddr *)&req->src, &salen) < 0) {
+		char buf[BUFSIZ];
+		req->udp = 1;
+		salen = sizeof(req->src);
+		recvfrom(req->sock, buf, sizeof(buf), MSG_PEEK, (struct sockaddr *)&req->src, &salen);
+	}
+	salen = sizeof(req->dst);
+	getsockname(req->sock, (struct sockaddr *)&req->dst, &salen);
 
 	req->error = 0;
 	req->lineno = 0;
@@ -705,15 +710,20 @@ int hosts_access(struct request_info *req)
 int hosts_ctl(char *daemon, char *client_name, char *client_addr,
 	      char *client_user)
 {
-    struct request_info request;
+    struct request_info *request;
+    int ret;
 
-    request_init(&request, RQ_DAEMON, daemon, RQ_CLIENT_NAME, client_name,
-		    RQ_CLIENT_ADDR, client_addr, RQ_USER, client_user, RQ_END);
-    switch(hosts_access(&request)) {
-    case TCPD_ALLOW:
-	    return (TCPD_ALLOW);
-	    break;
-    default:
-	    return (TCPD_DENY);
-    }
+    request = request_init(RQ_DAEMON, daemon,
+		    RQ_CLIENT_NAME, client_name,
+		    RQ_CLIENT_ADDR, client_addr,
+		    RQ_USER, client_user,
+		    RQ_END);
+
+    if (request == NULL)
+	    return TCPD_DENY;
+
+    ret = hosts_access(request);
+
+    free(request);
+    return ret;
 }
