@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID("$Id: rfc1413.c,v 1.5 2000/05/15 12:38:55 misiek Exp $")
+RCSID("$Id: rfc1413.c,v 1.7 2001/10/17 10:22:54 wiget Exp $")
 
 #include <stdio.h>
 #include <setjmp.h>
@@ -79,7 +79,7 @@ char *rfc1413(
 	struct sockaddr_storage *remote,
 	char *user, int userlen, int rfc1413_timeout)
 {
-	int s;
+	int s, old_alarm;
 	char *p, *ret = NULL;
 	struct addrinfo hints, *rres, *lres;
 	char buf[STRINGLEN];
@@ -119,7 +119,7 @@ char *rfc1413(
 		return NULL;
 	timeout_happen = 0;
 	signal(SIGALRM, timeout);
-	alarm(rfc1413_timeout);
+	old_alarm = alarm(rfc1413_timeout);
 	if (bind(s, lres->ai_addr, lres->ai_addrlen) == 0
 		  && connect(s, rres->ai_addr, rres->ai_addrlen) >= 0
 		  && writes(s, buf)
@@ -133,7 +133,7 @@ char *rfc1413(
 			ret = user;
 			
 		}
-	alarm(0);
+	alarm(old_alarm);
 	close(s);
 	signal(SIGALRM, SIG_IGN);
 	freeaddrinfo(lres);
