@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID(PKG_VER "$Id: tcpd.c,v 1.4 2000/02/28 12:48:25 misiek Exp $")
+RCSID(PKG_VER "$Id: tcpd.c,v 1.7 2001/10/17 14:11:54 wiget Exp $")
 
 #include <stdio.h>
 #include <unistd.h>
@@ -42,7 +42,7 @@ RCSID(PKG_VER "$Id: tcpd.c,v 1.4 2000/02/28 12:48:25 misiek Exp $")
 
 int main(int argc, char *argv[])
 {
-	struct request_info req;
+	struct request_info *req;
 	char path[TCPD_STRINGLENL];
 	int ret;
 #ifdef DEBUG
@@ -67,8 +67,7 @@ int main(int argc, char *argv[])
 	(void) openlog(argv[0], LOG_PID);
 #endif /* !LOG_MAIL */
 
-	request_init(&req, RQ_FILE, 0, RQ_DAEMON, argv[0], RQ_END);
-	fromhost(&req);
+	req = request_init(RQ_FILE, 0, RQ_DAEMON, argv[0], RQ_END);
 
 	ret = hosts_access(&req);
 
@@ -79,22 +78,25 @@ int main(int argc, char *argv[])
 	case TCPD_DENY:  rets = "deny "; break;
 	default:         rets = "xxxxx"; break;
 	}
-	tcpd_satostr(str, sizeof(str), &req.dst);
-	syslog(allow_severity, "%s@%s[%s] ", req.service, req.dst_name, str);
-	tcpd_satostr(str, sizeof(str), &req.src);
+	tcpd_satostr(str, sizeof(str), &req->dst);
+	syslog(allow_severity, "%s@%s[%s] ", req->service, req->dst_name, str);
+	tcpd_satostr(str, sizeof(str), &req->src);
 	syslog(allow_severity, "%s@%s[%s] ", req.user, req.src_name, str);
 
-	syslog(allow_severity, "%s lineno=%d error=%d syslog=%03x/%03x command=[%s]\n", rets, req.lineno, req.error, allow_severity, deny_severity, req.commands);
+	syslog(allow_severity, "%s lineno=%d error=%d syslog=%03x/%03x command=[%s]\n",
+			rets, req->lineno, req->error,
+			allow_severity, deny_severity, req->commands);
 #endif /* DEBUG */
 	
 	if (!ret)
-		refuse(&req);
-	tcpd_allowlog(&req);
-	if (tcpd_exec_option(&req) == TCPD_ERROR)
-		tcpd_refuse2(&req);
+		refuse(req);
+	tcpd_allowlog(req);
+	if (tcpd_exec_option(req) == TCPD_ERROR)
+		tcpd_refuse2(req);
 	(void)execv(path, argv);
 	syslog(allow_severity, "error: cannot execute %s: %m", path);
 	closelog();
-	clean_exit(&req);
+	clean_exit(req);
+	free_request(req);
 	exit(0);
 }
